/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;

/**
 * This is the main class of the Conient client.
 *
 * This sets up the control panel, the data panel
 * and its own display.
 *
 * @author  $Author: tdb $
 * @version $Id: Conient.java,v 1.36 2004/08/01 10:40:05 tdb Exp $
 */
public class Conient extends JFrame {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.36 $";

    /**
     * The initial width of the window
     */        
    private final int DEFAULT_WIDTH = 700;
    
    /**
     * The initial height of the window
     */
    private final int DEFAULT_HEIGHT = 600;
    
    /**
     * The default configuration fle
     * This can be specified on the command line
     */
    public static final String DEFAULT_CONFIG_FILE = "./etc/default.conf";
    
    /**
     * The time in seconds to display the splash
     * screen for
     */
    public static final int DISPLAY_SPLASH_TIME_SECONDS = 3;
    
//---STATIC METHODS---

    /**
     * The first method that is called.
     * Sets up the various panels
     *
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        // get the host from the command line if they gave it
        String host = null;
        String configFile = DEFAULT_CONFIG_FILE;
        if (args.length == 1) {
            configFile = args[0];
        }

        // start the configuration system
        Configuration.initialise(configFile);
 
        // the data display panel
        DataPanel data = new DataPanel();
        
        // the control panel
        ControlPanel control = new ControlPanel(data);
        
        // the main frame (passed the two panels)
        Conient client = new Conient(data, control);
        conientFrame = (Frame) client;
        Conient.addMessage("Conient {an i-scream Client} (c) 2001 The i-scream Project (http://www.i-scream.org)");

        Conient.addMessage("Conient ready.");
        
    }
    
    /**
     * A static accessor, allowing components of the system
     * to get hold of the root frame of the system.
     *
     * see conientFrame attribute for details.
     *
     * @return the root Conient frame
     */
    public static Frame getFrame() {
        return conientFrame;
    }
    
//---CONSTRUCTORS---

    /**
     * Creates a new Swing Client Frame
     */
    private Conient(JSplitPane data, ControlPanel control) {
        // set up the Frame
        super("Conient {an i-scream Client}");
        setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);
        setJMenuBar(control.getMenuBar());
        
        // set what happens when the X in the corner is clicked        
        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {System.exit(0);}
        });

        // add the control pane to the top border
        getContentPane().add(control, "North");
        // add the data pane to the centre        
        getContentPane().add(data);

        // setup the status panel        
        JPanel bottom = new JPanel();
        bottom.setLayout(new BorderLayout());
        _messages.setEditable(false);
        _messages.setRows(3);
        JScrollPane messagesPane = new JScrollPane(_messages);
        messagesPane.setBorder(new TitledBorder(new LineBorder(new Color(0, 0, 102)), " Messages "));
        bottom.add(messagesPane, "Center");
        JPanel statusPanel = new JPanel();
        JPanel linkPanel = new JPanel();
        linkPanel.setLayout(new GridLayout(1,2));
        linkPanel.add(_controlStatus);
        linkPanel.add(_dataStatus);
        statusPanel.setLayout(new GridLayout(2,1));
        statusPanel.add(linkPanel);
        statusPanel.add(_queueStatus);
        bottom.add(statusPanel, "South");

        // add the status panel to the bottom border
        getContentPane().add(bottom, "South");
        
        // a nice icon for the window
        setIconImage((new ImageIcon(getClass().getResource("/resources/server.gif"))).getImage());

        // and just because we can, a silly splash screen
        // of the dudes that did this funky jibble
        Splash splash = new Splash();
        splash.show();
        // wait
        try {
            Thread.sleep(DISPLAY_SPLASH_TIME_SECONDS * 1000);
        } catch (InterruptedException e) {
            // don't do anything, we don't care
        }
        //loose the window
        splash.dispose();
        splash = null;        
        // show the main window
        show();
    }

//---PUBLIC METHODS---

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * Sets the control link status.
     *
     * @param status the message
     */
    public static void setControlStatus(String status) {
        _controlStatus.setText("Control Link: " + status);
        _controlStatus.repaint();
    }
    
    /**
     * Sets the data link status.
     *
     * @param status the message
     */
    public static void setDataStatus(String status) {
        _dataStatus.setText("Data Link: " + status);
        _dataStatus.repaint();
    }

    /**
     * Updates the queue status.
     *
     * @param currentQueue
     * @param numElements
     */
    public static void setQueueStatus(int currentQueue, int numElements) {
        _queueStatus.setText("Data Queue : " + currentQueue + " Packets Recieved : " + numElements);
        _queueStatus.repaint();
    }

    /**
     * Adds a system message to the messages list
     *
     * @param message the new message
     */
    public static void addMessage(String message) {
        _messages.insert(message + "\n", 0);
    }

//---ATTRIBUTES---
    
//---STATIC ATTRIBUTES---
    
    /**
     * Displays information about the data link
     */
    static JLabel _controlStatus = new JLabel("Control Link: Disconnected", JLabel.LEFT);
    {
        _controlStatus.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
    }
    
    /**
     * Displays information about the data link
     */
    static JLabel _dataStatus = new JLabel("Data Link: Disconnected", JLabel.LEFT);
    {
        _dataStatus.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
    }
    
    /**
     * Displays information about the inbound data queue.
     */
    static JLabel _queueStatus = new JLabel(" ", JLabel.LEFT);
    {
        _queueStatus.setBorder(new EtchedBorder(EtchedBorder.LOWERED));
    }
    
    /**
     * The place where system messages are written.
     */
    static JTextArea _messages = new JTextArea();
    {
        _messages.setLineWrap(true);
    }
    
    /**
     * Holds a reference to the root frame for Conient
     * This is only used by dialogs (specifically the configurationn
     * dialog) so that it can be modal, please use the accessor.
     */
    private static Frame conientFrame;
    
//---INNER CLASSES----

    /**
     * An inner class to display a splash screen
     */
    private class Splash extends JWindow {
        
        /**
         * Constructs a new splash screen
         */
        public Splash() {
            setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            JPanel splash = new JPanel();
            splash.setBackground(Color.black);
            JLabel image = new JLabel((new ImageIcon(getClass().getResource("/resources/i-scream-splash.gif"))));
            splash.add(image);
            setContentPane(splash);
            Dimension screen = getToolkit().getScreenSize();
            pack();
            setLocation((screen.width - getSize().width) / 2,
                        (screen.height - getSize().height) / 2);
            
            
        }
    }
}
