/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient;

//---IMPORTS---
import uk.org.iscream.cms.util.*;
import uk.org.iscream.cms.conient.datacomponents.*;
import java.util.HashMap;
import java.util.Set;
import java.util.Collections;
import java.util.Iterator;
import javax.swing.BoxLayout;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JFrame;
import javax.swing.Box;
import java.awt.event.*;
import java.awt.*;
import javax.swing.SwingUtilities;
import javax.swing.ImageIcon;

/**
 * A HostDisplayPanel is simply a JPanel containing
 * DataComponents which relate to various host
 * attributes.  Once created, it can be asked to 
 * update the data it displays by passing it an XML
 * packet for the host it displaying for.
 * This then updates all the DataComponents with the 
 * values in the packet.
 *
 * @author  $Author: tdb $
 * @version $Id: HostDisplayPanel.java,v 1.53 2005/04/16 15:08:02 tdb Exp $
 */
public class HostDisplayPanel extends JPanel {

//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.53 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

    /**
     * Creates a new Host Display Panel and adds
     * all the appropriate DataComponents
     */
    public HostDisplayPanel(String hostName) {
        super();
        
        // set up the Panel
        setLayout(new BorderLayout());
        _hostName = hostName;
        
        // check to config to see if we want extra data or not
        if(Configuration.getInstance().getProperty("displayExtraData").equals("1")) {
            _extraData = true;
        }
        
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        c.fill = GridBagConstraints.HORIZONTAL;
        c.anchor = GridBagConstraints.NORTH;
        c.weighty = 1.0;
        _center.setLayout(gridbag);
        
        JPanel centerHolder = new JPanel();
        centerHolder.add(_center);
        add(centerHolder, "Center");
        add(_north, "North");

        addDataComponent(new StringDataComponent("Packet Type", "packet.attributes.type"));
        
        final StringDataComponent machineName = new StringDataComponent("Host Name", "packet.attributes.machine_name");
        addDataComponent(machineName );
        final StringDataComponent ip = new StringDataComponent("IP Address", "packet.attributes.ip");
        addDataComponent(ip);
        final UptimeDataComponent uptime = new UptimeDataComponent("Uptime", "packet.os.uptime");
        addDataComponent(uptime);
        final StringDataComponent seq_no = new StringDataComponent("Packets since host started", "packet.attributes.seq_no");
        addDataComponent(seq_no);
        
        final StringDataComponent osName = new StringDataComponent("Operating System", "packet.os.name");
        addDataComponent(osName );
        final StringDataComponent osVer = new StringDataComponent("Operating System Version", "packet.os.version");
        addDataComponent(osVer);
        final StringDataComponent osRelease = new StringDataComponent("Operating System Release", "packet.os.release");
        addDataComponent(osRelease);
        final StringDataComponent osSysName = new StringDataComponent("System Name", "packet.os.sysname");
        addDataComponent(osSysName );
        final StringDataComponent netbiosName = new StringDataComponent("NetBIOS Name", "packet.os.netbios_name");
        addDataComponent(netbiosName );
        final StringDataComponent osPlatform = new StringDataComponent("System Architecture", "packet.os.platform");
        addDataComponent(osPlatform );
                
        JButton info = new JButton("Platform Information");
        info.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                final JFrame infoFrame = new JFrame("Host platform information for - " + _hostName);
                Box content = Box.createVerticalBox();
                content.add(machineName);
                content.add(ip);
                content.add(uptime);
                content.add(seq_no);
                content.add(osName);
                content.add(osVer);
                content.add(osRelease);
                content.add(osSysName);
                content.add(netbiosName);
                content.add(osPlatform);
                infoFrame.getContentPane().add(content, "Center");
                JButton close = new JButton("Close Window");
                close.addActionListener(new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        infoFrame.dispose();
                    }
                });
                infoFrame.getContentPane().add(close, "South");
                infoFrame.setIconImage((new ImageIcon(getClass().getResource("/resources/server.gif"))).getImage());
                infoFrame.pack();
                infoFrame.setVisible(true);
            }
        });
        try {
            _dataTimer = new PacketTimer("Next Data", Integer.parseInt(Configuration.getInstance().getServerProperty("Host." + hostName, "Host.UDPUpdateTime")));
        } catch (NumberFormatException e) {
            throw new NumberFormatException("unable to obtain a valid value for Host.UDPUpdateTime");
        }
        _north.setLayout(new GridLayout(3,1));
        _north.add(new JLabel("Showing data from - " + hostName, JLabel.CENTER));
        _north.add(_dataTimer);
        _north.add(info);


        JPanel date = new JPanel();
        addVisibleDataComponent(date, new DateDataComponent("Host Time", "packet.attributes.date"));
        c.gridy = 0;
        gridbag.setConstraints(date, c);
        _center.add(date);
        
        // holder for labels
        JLabel label;

        label = new javax.swing.JLabel("--- CPU ---", JLabel.CENTER);
        c.gridy = 1;
        gridbag.setConstraints(label, c);
        _center.add(label);
        
        // add all the DataComponents that we know about
        JPanel cpu = new JPanel();
        cpu.setLayout(new BoxLayout(cpu, BoxLayout.Y_AXIS));
        addVisibleDataComponent(cpu, new CPUDataComponent("CPU Idle", "packet.cpu.idle"));
        addVisibleDataComponent(cpu, new CPUDataComponent("CPU User", "packet.cpu.user"));
        addVisibleDataComponent(cpu, new CPUDataComponent("CPU Kernel", "packet.cpu.kernel"));
        addVisibleDataComponent(cpu, new CPUDataComponent("CPU i/o wait", "packet.cpu.iowait"));
        addVisibleDataComponent(cpu, new CPUDataComponent("CPU swapping", "packet.cpu.swap"));
        c.gridy = 2;
        gridbag.setConstraints(cpu, c);
        _center.add(cpu);
        
        label = new javax.swing.JLabel("--- Load ---", JLabel.CENTER);
        c.gridy = 3;
        gridbag.setConstraints(label, c);
        _center.add(label);
        
        Box loadData = Box.createHorizontalBox();
        addVisibleDataComponent(loadData, new ProcessesDataComponent("1 minute", "packet.load.load1"));
        addVisibleDataComponent(loadData, new ProcessesDataComponent("5 minutes", "packet.load.load5"));
        addVisibleDataComponent(loadData, new ProcessesDataComponent("15 minutes", "packet.load.load15"));
        
        c.gridy = 4;
        gridbag.setConstraints(loadData, c);
        _center.add(loadData);
        
        label = new javax.swing.JLabel("--- Processes ---", JLabel.CENTER);
        c.gridy = 5;
        gridbag.setConstraints(label, c);
        _center.add(label);
        
        Box processesData = Box.createHorizontalBox();
        addVisibleDataComponent(processesData, new ProcessesDataComponent("Total", "packet.processes.total"));
        addVisibleDataComponent(processesData, new ProcessesDataComponent("Running", "packet.processes.cpu"));
        addVisibleDataComponent(processesData, new ProcessesDataComponent("Sleeping", "packet.processes.sleeping"));
        addVisibleDataComponent(processesData, new ProcessesDataComponent("Stopped", "packet.processes.stopped"));
        addVisibleDataComponent(processesData, new ProcessesDataComponent("Zombie", "packet.processes.zombie"));
        c.gridy = 6;
        gridbag.setConstraints(processesData, c);
        _center.add(processesData);

        label = new javax.swing.JLabel("--- Memory ---", JLabel.CENTER);
        c.gridy = 7;
        gridbag.setConstraints(label, c);
        _center.add(label);
        
        // these next two are passed the non-visible datacomponents
        JPanel memory = new JPanel();
        memory.setLayout(new BoxLayout(memory, BoxLayout.Y_AXIS));        
        addVisibleDataComponent(memory, new StorageDataComponent("Memory in use", "packet.memory.free", "packet.memory.total", "Mb", 1024*1024));
        addVisibleDataComponent(memory, new StorageDataComponent("Swap in use", "packet.swap.free", "packet.swap.total", "Mb", 1024*1024));
        addVisibleDataComponent(memory, new IODataComponent("Pages in/out", "packet.pages.pageins", "packet.pages.pageouts"));
        // theses next two are DataComponents, but not VisibleDataComponents
        // this is because they hold the value but do NOT display it
        // just so we ignore the data, because the StorageDataComponent handles it
        addDataComponent(new StringDataComponent("packet.pages.pageouts", "packet.pages.pageouts"));
        addDataComponent(new StringDataComponent("packet.memory.total", "packet.memory.total"));
        addDataComponent(new StringDataComponent("packet.memory.used", "packet.memory.used"));
        // maybe this one should be used somewhere?
        addDataComponent(new StringDataComponent("packet.memory.cache", "packet.memory.cache"));
        addDataComponent(new StringDataComponent("packet.swap.total", "packet.swap.total"));
        addDataComponent(new StringDataComponent("packet.swap.used", "packet.swap.used"));
        c.gridy = 8;
        gridbag.setConstraints(memory, c);
        _center.add(memory);
        
        label = new javax.swing.JLabel("--- Network ---", JLabel.CENTER);
        c.gridy = 9;
        gridbag.setConstraints(label, c);
        _center.add(label);
        c.gridy = 10;
        gridbag.setConstraints(_net, c);
        _center.add(_net);

        label = new javax.swing.JLabel("--- Users ---", JLabel.CENTER);
        c.gridy = 11;
        gridbag.setConstraints(label, c);
        _center.add(label);
        
        JPanel users = new JPanel();
        users.setLayout(new BoxLayout(users, BoxLayout.Y_AXIS));
        addVisibleDataComponent(users, new StringDataComponent("Number of users", "packet.users.count"));
        UsersDataComponent userList = new UsersDataComponent("User List", "packet.users.list");
        addVisibleDataComponent(users, userList);
        c.gridy = 12;
        gridbag.setConstraints(users, c);
        _center.add(users);
        
        label = new javax.swing.JLabel("--- Disks ---", JLabel.CENTER);
        c.gridy = 13;
        gridbag.setConstraints(label, c);
        _center.add(label);
        c.gridy = 14;
        gridbag.setConstraints(_disks, c);
        _center.add(_disks);
        c.gridy = 15;
        gridbag.setConstraints(_diskio, c);
        _center.add(_diskio);

        label = new javax.swing.JLabel("--- System Services ---", JLabel.CENTER);
        c.gridy = 16;
        gridbag.setConstraints(label, c);
        _center.add(label);
        c.gridy = 17;
        gridbag.setConstraints(_services, c);
        _center.add(_services);

        // everything else we get is "extra", but we may still want to display it        
        if (_extraData) {
            label = new javax.swing.JLabel("--- Extra Data ---", JLabel.CENTER);
            c.gridy = 18;
            gridbag.setConstraints(label, c);
            _center.add(label);
            c.gridy = 19;
            gridbag.setConstraints(_extra, c);
            _center.add(_extra);
        }
    }

//---PUBLIC METHODS---

    /**
     * This method takes an XMLPacket containing 
     * information about the host this HostDisplayPanel
     * is currently displaying.  It then updates all of the
     * components with the new values.
     *
     * It also adds a standard StringDataComponent, should
     * an attribute come up that there are no DataComponents
     * associated with.
     *
     * @param packet the XML packet
     * @return if the update was sucessful
     */
    public boolean updateHost(XMLPacket packet) {
        // set that everything is ok so far
        boolean displaySucessful = true;
        
        // must be a normal data packet, update the display
        if((packet.getParam("packet.attributes.type")).equals("data")) {
            displaySucessful = processPacket(packet);
            _dataTimer.reset();
            if(_dataTimerThread == null){
                _dataTimerThread = new Thread(_dataTimer);
                _dataTimerThread.start();
            }
        }
        return displaySucessful;        
    }
    
//---PRIVATE METHODS---

    /**
     * Process an incoming packet, updates the components.
     * If extra data is configured to be displayed, it will
     * create a new StringDataComponent to display the data.
     *
     * @param packet the packet to process
     *
     * @return if the proceesing was successful
     */
    private boolean processPacket(XMLPacket packet) {
        // iterate over the packets data
        boolean displaySucessful = true;
        Set packetSet = packet.getSet();
        Iterator i = packetSet.iterator();
        while (i.hasNext()) {
            String dataKey = (String) i.next();

            // if there are no components looking after this data
            // create a new StringDataComponent for it
            if(!_components.containsKey(dataKey)) {
                // check if its a disk drive, if it is then we need to deal with it
                if(dataKey.startsWith("packet.disk.p")) {
                    createDiskPanel(dataKey, packet);
                // check for diskio stats
                } else if(dataKey.startsWith("packet.diskio.p")) {
                    createDiskIOPanel(dataKey, packet);
                // check for network stats
                } else if(dataKey.startsWith("packet.net.p")) {
                    createNetworkPanel(dataKey, packet);
                // !check for services here! to display
                } else if(dataKey.startsWith("packet.services.")) {
                    createServicesPanel(dataKey);
                // otherwise we don't know about it...
                } else if (_extraData) {                    
                    addVisibleDataComponent(_extra, new StringDataComponent(dataKey, dataKey));
                }
            }
            
            // try and update the component, if it doesn't like what it gets
            // warn the user and set that this was an unsucessful display
            if(_components.containsKey(dataKey)) {
                
                try {
                    ((DataComponent) _components.get(dataKey)).setValue(packet);
                } catch (DataFormatException e) {
                    Conient.addMessage("WARNING{host display}: " + e.getMessage());
                    displaySucessful = false;
                }
            }
        }
        return displaySucessful;
    }
    
    /**
     * Adds a DataComponent to the list of already
     * known data components. The list is aware of
     * what packet attributes that a DataComponent
     * is looking after.
     *
     * @param ref the packet attribute that this Component looks after
     * @param dataComponent the actual DataComponent
     */
    private void addDataComponent(DataComponent dataComponent) {
        _components.put(dataComponent.getPacketAttribute(), dataComponent);
    }

    /**
     * Adds a DataComponent to the list of already
     * known data components. The list is aware of
     * what packet attributes that a DataComponent
     * is looking after.
     *
     * Used for VISIBLE data components, as they are added to the display.
     *
     * @param ref the packet attribute that this Component looks after
     * @param dataComponent the actual DataComponent
     */
    public void addVisibleDataComponent(Container holder, VisibleDataComponent dataComponent) {
        _components.put(((DataComponent) dataComponent).getPacketAttribute(), (DataComponent) dataComponent);
        SwingSafeAdd task = new SwingSafeAdd(holder, dataComponent);
        SwingUtilities.invokeLater(task);
    }

    /**
     * Creates the disk components and adds them
     * do the list of components.  It also displays
     * any relavant component.
     *
     * When a packet attribute is new to the system
     * and when it starts "packet.disk.p" then this 
     * method is called.  Thus it then accounts for
     * all attributes relating to that new disk.
     *
     * @param attribute the name of the attribute first encountered
     * @param packet the XMLPacket the data is being pulled from
     */
    private void createDiskPanel(String attribute, XMLPacket packet) {
        String diskNumber = "";
        
        // pos is after "packet.disk.p"
        int pos = 13;
        while (attribute.charAt(pos) != '.') {
            diskNumber = diskNumber + attribute.charAt(pos);
            pos++;
        }
        String devAttribute = "packet.disk.p" + diskNumber + ".attributes.name";
        String mountAttribute = "packet.disk.p" + diskNumber + ".attributes.mount";
        String totalAttribute = "packet.disk.p" + diskNumber + ".attributes.total";
        
        // the only visible component
        String availAttribute = "packet.disk.p" + diskNumber + ".attributes.avail";
        DiskDataComponent disk = new DiskDataComponent("Disk", availAttribute, totalAttribute, mountAttribute, devAttribute , "Mb", 1024*1024);
        addVisibleDataComponent(_disks, disk);
        
        // currently this data isn't useful, so we don't display it
        // but we add it so that it is forgotten about
        addDataComponent(new StringDataComponent(devAttribute, devAttribute));
        addDataComponent(new StringDataComponent(mountAttribute, mountAttribute));
        addDataComponent(new StringDataComponent(totalAttribute, totalAttribute));
        
        // add any further bits of data about this disk so we
        // don't try and do something with them later
        // -- this is not that tidy ;)
        Set packetSet = packet.getSet();
        Iterator i = packetSet.iterator();
        while (i.hasNext()) {
            String dataKey = (String) i.next();
            if(dataKey.startsWith("packet.disk.p" + diskNumber +".") && !_components.containsKey(dataKey)) {
                addDataComponent(new StringDataComponent(dataKey, dataKey));
            }
        }
    }
    
    /**
     * Creates the diskio components and adds them
     * do the list of components.
     *
     * When a packet attribute is new to the system
     * and when it starts "packet.diskio.p" then this 
     * method is called.  Thus it then accounts for
     * all attributes relating to that new diskio.
     *
     * @param attribute the name of the attribute first encountered
     * @param packet the XMLPacket the data is being pulled from
     */
    private void createDiskIOPanel(String attribute, XMLPacket packet) {
        String diskNumber = "";
        
        // pos is after "packet.diskio.p"
        int pos = 15;
        while (attribute.charAt(pos) != '.') {
            diskNumber = diskNumber + attribute.charAt(pos);
            pos++;
        }
        
        // grab the attributes
        String nameAttribute = "packet.diskio.p" +  diskNumber + ".attributes.name";
        String wbytesAttribute = "packet.diskio.p" +  diskNumber + ".attributes.wbytes";
        String rbytesAttribute = "packet.diskio.p" +  diskNumber + ".attributes.rbytes";
        
        // use an IODataComponent to display the diskio
        IODataComponent diskio = new IODataComponent(packet.getParam(nameAttribute), rbytesAttribute, wbytesAttribute);
        addVisibleDataComponent(_diskio, diskio);
        
        // these attributes are also used, so we hold on to them
        // (as a side note, this way of "holding on" to attributes sucks ;)
        addDataComponent(new StringDataComponent(nameAttribute, nameAttribute));
        addDataComponent(new StringDataComponent(wbytesAttribute, wbytesAttribute));
    }
    
    /**
     * Creates the netwrok components and adds them
     * do the list of components.
     *
     * When a packet attribute is new to the system
     * and when it starts "packet.net.p" then this 
     * method is called.  Thus it then accounts for
     * all attributes relating to that network device.
     *
     * @param attribute the name of the attribute first encountered
     * @param packet the XMLPacket the data is being pulled from
     */
    private void createNetworkPanel(String attribute, XMLPacket packet) {
        String netNumber = "";
        
        // pos is after "packet.net.p"
        int pos = 12;
        while (attribute.charAt(pos) != '.') {
            netNumber = netNumber + attribute.charAt(pos);
            pos++;
        }
        
        // grab the attributes
        String nameAttribute = "packet.net.p" +  netNumber + ".attributes.name";
        String txAttribute = "packet.net.p" +  netNumber + ".attributes.tx";
        String rxAttribute = "packet.net.p" +  netNumber + ".attributes.rx";
        
        // use an IODataComponent to display the diskio
        IODataComponent net = new IODataComponent(packet.getParam(nameAttribute), rxAttribute, txAttribute);
        addVisibleDataComponent(_net, net);
        
        // these attributes are also used, so we hold on to them
        addDataComponent(new StringDataComponent(nameAttribute, nameAttribute));
        addDataComponent(new StringDataComponent(txAttribute, txAttribute));
    }
    
    /**
     * Creates the servicecheck components and adds them
     * do the list of components.  It also displays
     * any relavant component.
     *
     * When a packet attribute is new to the system
     * and when it starts "packet.services." then this 
     * method is called.  Thus it then accounts for
     * all attributes relating to that service.
     *
     * @param attribute the name of the attribute first encountered
     */
    private void createServicesPanel(String attribute) {
        String serviceType = "";
        
        // pos is after "packet.services."
        int pos = 16;
        while (attribute.charAt(pos) != '.') {
            serviceType = serviceType + attribute.charAt(pos);
            pos++;
        }
        
        String statusAttribute = "packet.services." + serviceType + ".attributes.status";
        // we hide this as the service data component uses this attribute
        addDataComponent(new StringDataComponent(serviceType + " service status", statusAttribute));
                
        ServiceDataComponent service = new ServiceDataComponent(serviceType, "packet.services." + serviceType + ".attributes.message", statusAttribute);
        addVisibleDataComponent(_services, service);
    }

//---ACCESSOR/MUTATOR METHODS---

//---ATTRIBUTES---
        
    /**
     * The components that we already know about
     * connected with the attributes they care about
     * are stored here.
     */
    private HashMap _components = new HashMap();
    
    /**
     * The north panel, currently used for the platform info button.
     */
    private final JPanel _north = new JPanel();

    /**
     * The centre panel, where all the data is displayed.
     */
    private final JPanel _center = new JPanel();
    
    /**
     * The countdown timer for normal data packets
     */
    private final PacketTimer _dataTimer;
    
    /**
     * A place for the data timer thread
     */
    private Thread _dataTimerThread = null;
    
    /**
     * The hostname this instance is keeping data for
     */
    private final String _hostName;
    
    /**
     * The disks panel
     */
    private final JPanel _disks = new JPanel();
    {
        _disks.setLayout(new BoxLayout(_disks, BoxLayout.Y_AXIS));
    }
    
    /**
     * The diskio panel
     */
    private final JPanel _diskio = new JPanel();
    {
        _diskio.setLayout(new BoxLayout(_diskio, BoxLayout.Y_AXIS));
    }
    
    /**
     * The network panel
     */
    private final JPanel _net = new JPanel();
    {
        _net.setLayout(new BoxLayout(_net, BoxLayout.Y_AXIS));
    }
    
    /**
     * The extra panel, used to display unknown data in the packet
     */
    private final JPanel _extra = new JPanel();
    {
        _extra.setLayout(new BoxLayout(_extra, BoxLayout.Y_AXIS));
    }
    
    /**
     * The services panel, currently used to display host service information
     * from heartbeat packets.
     */
    private final JPanel _services = new JPanel();
    {
        _services.setLayout(new BoxLayout(_services, BoxLayout.Y_AXIS));
    }
    
    /**
     * Whether we a worrying about "extra" data
     * by default we don't, but we look in the configuration
     * in the constructor.
     */
    private boolean _extraData = false;
    
//---STATIC ATTRIBUTES---
    
}
