/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This interface is used by all
 * components that will be used to display
 * inbound data.  The HostDisplayPanel class
 * uses it as a generic way to tell a specific
 * class that holds data to update its data with
 * a given packet.
 *
 * A related class is the VisibleDataComponent
 * abstract class, this is specifically for components
 * that will display data and thus implements this
 * interface.
 *
 * @author  $Author: tdb $
 * @version $Id: DataComponent.java,v 1.13 2004/08/01 10:40:10 tdb Exp $
 */
public interface DataComponent {
    
//---FINAL ATTRIBUTES---

    /**
     * The current CVS revision of this class
     */
    public final String REVISION = "$Revision: 1.13 $";
    
//---STATIC METHODS---
    
//---CONSTRUCTORS---

//---PUBLIC METHODS---

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---

    /**
     * This takes an XMLPacket so that the reading
     * component can extract the data it needs to update
     * the display.
     *
     * @param packet the packet
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException;

    /**
     * Returns the string showing the packet
     * attribute that the component is looking after
     *
     * @return the packet reference
     */
    public String getPacketAttribute();

//---ATTRIBUTES---

//---STATIC ATTRIBUTES---

}
