/* 
 * i-scream central monitoring system
 * http://www.i-scream.org
 * Copyright (C) 2000-2002 i-scream
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

//---PACKAGE DECLARATION---
package uk.org.iscream.cms.conient.datacomponents;

//---IMPORTS---
import javax.swing.JLabel;
import java.awt.GridLayout;
import java.util.Date;
import java.text.DateFormat;
import javax.swing.SwingUtilities;
import uk.org.iscream.cms.util.XMLPacket;

/**
 * This is DataComponent specifically for converting
 * time since epoc dates into a nice format.
 *
 * As this is essentially just string data we just
 * extend StringDataComponent, intercepting the value
 * to convert it.
 *
 * @author  $Author: tdb $
 * @version $Id: DateDataComponent.java,v 1.14 2004/08/01 10:40:10 tdb Exp $
 */
public class DateDataComponent extends StringDataComponent {

//---FINAL ATTRIBUTES---

//---STATIC METHODS---

//---CONSTRUCTORS---

    /**
     * Creates the component with a friendly name to be 
     * used as label, but as we're a very basic
     * extension of StringDataComponent, we just construct
     * that.
     *
     * @param name the friendly name
     * @param attribute the data attribute we look after
     */
    public DateDataComponent(String name, String attribute) {
        super(name, attribute);
    }

//---PUBLIC METHODS---

//---PRIVATE METHODS---

//---ACCESSOR/MUTATOR METHODS---
     
    /**
     * This takes the packet to obtain the value from, it then performs all
     * approriate conversions and adds this class to the Swing Event
     * Dispatching queue.
     *
     * In this case all we do is change string and pass it to our
     * super class for displaying.  Note that we need to *1000
     * as the data standard is for time in seconds, but java likes
     * milliseconds.
     *
     * @param packet the XMLPacket to get the data from
     * @throws DataFormatException if there was a problem converting the data for display
     */
    public void setValue(XMLPacket packet) throws DataFormatException {
        String value = packet.getParam(_attribute);
        try {
            if(!_ourCache.equals(value)) {
                _ourCache = value;
                _cache = DateFormat.getDateTimeInstance(DateFormat.SHORT, DateFormat.MEDIUM).format(new Date(Long.parseLong(value) * 1000));
                SwingUtilities.invokeLater(this);
            }
        } catch (Exception e) {
            throw new DataFormatException(value + " is an invalid data type for " + toString());
        }
    }        

//---ATTRIBUTES---

    /**
     * Remembers what the last value was, so we
     * only update if we have to.
     */    
    String _ourCache = "";

//---STATIC ATTRIBUTES---

}
